(***************************************************************************
Author:    Germano Rossi, Verona University, Italy (c) All rights reserved.
Internet:  germano@chiostro.univr.it
Program:   clocheap.pas
Version:   1.6
Status:    Exp
Date:      1995-01-22 16:24:33-0800
Nickname:  
***************************************************************************)
{$I APP.DEF}

unit ClocHeap;

{
ClocHeap unit contains THeapView, TClockView, TClockHeapApplication
and a dialog to choose desktop options (hdDeskOptions).

This unit is intended for Turbo Pascal 7.0 with Turbo Vision
2.0. The source is free and comes with no warranties whatsoever.

I hope that you find this unit useful.

Copyright
---------
You may use this unit for free and pass it on to the other persons as
long as you clearly state, in the header as well as in the code, all
changes made to the original code.

You may not pass along compiled versions of the source, without this
source file. But, you are allowed to use this unit in your distributed
non commercial programs.

You may also upload this package, unmodified, to bulletin boards or
other public access places.

There must be no charge associated with this package, though you may
include the unmodified package as a whole in shareware/freeware
compilations if only a nominal amount is charged for the compilation
disk.

This unit includes:
----------------------

A TClockView object that display date and/or time in the upper right
corner (22 char max).

An THeapView object that display MemAvail memory amount in the lower
right corner (8 char max, 95 Mb maximum).

A TClockHeapApplication object.

A hdDeskView dialog to choose desktop look (included 25/43/50 screen mode).
}

{$F+,O+,S-,Q-,V-,X+}

interface

uses Dos, Objects, Views, App, Drivers, Dialogs;

const
  cmDesk = 5000;
  cmScreen = 5001;
  cmValueChanged = 65535;
  hcVideoOptions = 5000;
  hcColorOptions = 5001;
  hcDeskOptions  = 5002;
  hcDateOptions  = 5003;
  hcSeparatorsOptions = 5004;
  DateCentury : boolean = True;

const
  roHeap  : Boolean = true;      { If true display Heap on last row,
                                   if false not; default is True }
  roClock : Byte = 1;            { Display Time (1), Date (2), Both (3) or
                                   Nothing (0) on the first menu row;
                                   default is 1 }
  roVideo : Boolean = false;     { If true use 43/50 rows screen mode;
                                   default is 25 rows }
  roColor : Word = 0;            { Use Color (0), Black & White (1) or
                                   Monochrome (2) palette }
  roDate  : Word = 0;            { Date format: MDY (0), DMY (1) or YMD (2) }
  roSep   : Word = 2;            { Date Separator: '.' (0), '/' (1), '-'
                                   (2) }

type
  PHeapView = ^THeapView;
  THeapView = object(TView)
    FreeMem : LongInt;                        { Free memory in the heap }
    constructor Init(var Bounds: TRect);
    destructor Done; virtual;
    procedure Draw; virtual;
    function GetPalette: PPalette; virtual;
    procedure Update;
  end;

  PClockView = ^TClockView;
  TClockView = object(TView)
    Refresh       : Byte;
    LastTime      : DateTime;
    TimeStr       : string[10];
    DayOfWeek     : word;
    DateStr       : string [11];            { space for dd-Mmm-yy }
    DayStr        : string [3];             { space for Ddd }
    constructor Init(var Bounds: TRect);
    procedure Draw; virtual;
    function FormatTimeStr(DT: DateTime): String; virtual;
    function DateToStr(DT: DateTime): String;
    procedure Update; virtual;
  end;

  PClockHeapApplication = ^TClockHeapApplication;
  TClockHeapApplication = object(TApplication)
    constructor Init;
    destructor Done; virtual;
    procedure Idle; virtual;
    {$IFDEF ScreenSaverGroothuis}
    procedure GetEvent(var Event:TEvent);virtual;
    {$ENDIF}
    procedure HandleEvent(var Event: TEvent); virtual;

  end;

procedure hdDeskView;

procedure AddHeap;
procedure DumpHeap;
procedure UpdateHeap;

procedure AddClock;
procedure DumpClock;
procedure UpdateClock;

implementation

uses Crt, Memory
  {$IFDEF ScreenSaverGroothuis}
  , Scrsav
  {$ENDIF}
  ;

const
  ViewHeap : PHeapView = Nil;
  Clock    : PClockView = Nil;
  TmpStr   : string[8] = '        ';

{$IFDEF ScreenSaverGroothuis}
var
  ScreenSaver: PScreenSaver;
{$ENDIF}

type
  PSelLabel = ^TSelLabel;
  TSelLabel = object(TLabel)
    procedure HandleEvent(var Event: TEvent); virtual;
  end;
  PSelRadioButtons = ^TSelRadioButtons;
  TSelRadioButtons = object(TRadioButtons)
    constructor Init(var Bounds: TRect; AStrings: PSItem);
    procedure HandleEvent(var Event: TEvent); virtual;
  end;
  PSelCheckBoxes = ^TSelCheckBoxes;
  TSelCheckBoxes = object(TCheckBoxes)
    procedure HandleEvent(var Event: TEvent); virtual;
  end;

procedure TSelLabel.HandleEvent(var Event: TEvent);
begin
  if (Link <> nil) and (Link^.Options and ofSelectable = 0) then
    Exit;
end;

constructor TSelRadioButtons.Init(var Bounds: TRect; AStrings: PSItem);
begin
  inherited Init(Bounds, AStrings);
  EventMask := EventMask or evBroadcast;
  if roClock >= 3 then
    Options := Options and ofSelectable
  else
    Options := Options and not ofSelectable;
end;

procedure TSelRadioButtons.HandleEvent(var Event: TEvent);
type
  PWord = ^Word;
begin
  inherited HandleEvent(Event);
  if (Event.What = evBroadcast) and (Event.Command = cmValueChanged) then begin
    if PWord(Event.InfoPtr)^ and 4 = 4 then
      Options := Options or ofSelectable
    else
      Options := Options and not ofSelectable;
    DrawView;
  end;
end;

procedure TSelCheckBoxes.HandleEvent(var Event: TEvent);
var Temp: word;
begin
  Temp := Value;
  inherited HandleEvent(Event);
  if (Temp and 4 <> Value and 4) then
    Message(Owner, evBroadcast, cmValueChanged, @Value);
end;

{$IFDEF English}
{$I deskv-us.src}
{$ENDIF}
{$IFDEF Italian}
{$I deskv-it.src}
{$ENDIF}

{------ Heap Window object ----------}

constructor THeapView.Init(var Bounds: TRect);
begin
  inherited Init(Bounds);
  FreeMem := 0;
  GrowMode := gfGrowAll;
end;

destructor THeapView.Done;
begin
  inherited Done;
end;

procedure THeapView.Draw;
var
  S: string[8];
begin
  if roHeap then begin
    FreeMem := MemAvail;
    FormatStr(S, '%8d', FreeMem);
    if FreeMem > 100000 then
      WriteStr(0, 0, S, 1)
    else if FreeMem > 50000 then
      WriteStr(0, 0, S, 2)
    else begin
      WriteStr(0, 0, S, 3);
      Sound(3000);
      Delay((50000 - MemAvail) div 100);
      NoSound;
    end;
  end else begin
    WriteStr(0, 0, TmpStr, 1);
  end;
end;

function THeapView.GetPalette: PPalette;
const
  P: String[2] = #2#1;
begin
  GetPalette := @P;
end;

procedure THeapView.Update;
begin
  if (FreeMem <> MemAvail) then DrawView;
end;

{-------- ClockView Object --------}

function LeadingZero(w: Word): String;
var s: String;
begin
  Str(w:0, s);
  LeadingZero := Copy('00', 1, 2 - Length(s)) + s;
end;

constructor TClockView.Init(var Bounds: TRect);
begin
  inherited Init(Bounds);
  FillChar(LastTime, SizeOf(LastTime), #$FF);
  TimeStr := '';
  DayOfWeek := 255;
  DateStr := '';
  DayStr := '';
  Refresh := 1;
end;

procedure TClockView.Draw;
var
  B: TDrawBuffer;
  C: Byte;
begin
  C := GetColor(2);
  MoveChar(B, ' ', C, Size.X);
  case roClock of
    1 : MoveStr(B, TmpStr + #32 + TimeStr, C);
    2 : MoveStr(B, TmpStr + #32 + DateStr, C);
    3 : MoveStr(B, DateStr + #32 + TimeStr, C);
  end;
  WriteLine(0, 0, Size.X, 1, B);
end;


procedure TClockView.Update;
var
  NewTime                   : DateTime;
  Hund                      : word;
begin
  with NewTime do
    GetTime(Hour , Min , Sec , hund);
  if Abs(NewTime.Sec - LastTime.sec) < Refresh then exit;
  with NewTime do
    GetDate(Year, Month, Day, DayOfWeek);
  LastTime := NewTime;
  DateStr := DateToStr(LastTime);
  TimeStr := FormatTimeStr(LastTime);
  if Odd(LastTime.Sec) then TimeStr[6] := #32;
  DrawView;
end;

function TClockView.FormatTimeStr(DT: DateTime): String;
begin
  with DT do
    FormatTimeStr := LeadingZero(Hour)+ ':'+ LeadingZero(Min) +
      ':' + LeadingZero(Sec);
end;

function TClockView.DateToStr(DT: DateTime): String;
const Sep : string[3] = './-';
begin
  with DT do begin
    if DateCentury then
      if Year > 2000 then
        Dec(Year, 2000)
      else
        Dec(Year, 1900);
    case roDate of
      0 : DateToStr := LeadingZero(Month) + Sep[roSep] + LeadingZero(Day) +
          Sep[roSep] + LeadingZero(Year);
      1 : DateToStr := LeadingZero(Day) + Sep[roSep] + LeadingZero(Month) +
          Sep[roSep] + LeadingZero(Year);
      2 : DateToStr := LeadingZero(Year) + Sep[roSep] + LeadingZero(Month) +
          Sep[roSep] + LeadingZero(Day);
    end;
  end;
end;

{---------- ClockHeapApplication ------------------------------------------}

constructor TClockHeapApplication.Init;
begin
  inherited Init;
  AddClock;
  AddHeap;
  {$IFDEF ScreenSaverGroothuis}
  ScreenSaver := New(PScreenSaver, Init(0, 1));
  {$ENDIF}
end;

destructor TClockHeapApplication.Done;
begin
  {$IFDEF ScreenSaverGroothuis}
  Dispose(ScreenSaver, Done);
  {$ENDIF}
  DumpClock;
  DumpHeap;
  inherited Done;
end;

procedure TClockHeapApplication.Idle;
begin
  inherited Idle;
  UpdateClock;
  UpdateHeap;
  {$IFDEF ScreenSaverGroothuis}
  if ScreenSaver <> nil then ScreenSaver^.CountDown;
  {$ENDIF}
end;

{$IFDEF ScreenSaverGroothuis}
procedure TClockHeapApplication.GetEvent(var Event: TEvent);
begin
  Inherited GetEvent(Event);
  if Event.What <> evNothing then
    if ScreenSaver <> nil then
      if Event.What = evKeyDown then begin
        if ScreenSaver^.Saving then
          Event.What := evNothing;
        ScreenSaver^.HeartBeat;
      end else
        if Event.What and evMouse <> 0 then
          ScreenSaver^.HeartBeat;
end;
{$ENDIF}

procedure TClockHeapApplication.HandleEvent(var Event: TEvent);
begin
  inherited HandleEvent(Event);
  if Event.What = evCommand then begin
    case Event.Command of
      cmDesk : hdDeskView;
      {$IFDEF ScreenSaverGroothuis}
      cmScreen : ScreenSaver^.Options;
      {$ENDIF}
    end;
  end;
end;

{--------- Other procedures ------------------------------------------}

procedure AddHeap;
var
  R: TRect;
begin
  if Application = Nil then exit;
  Application^.GetExtent(R);
  R.A.X := R.B.X - 8;
  R.A.Y := R.B.Y - 1;
  ViewHeap := New(PHeapView, Init(R));
  Application^.Insert(ViewHeap);
end;

procedure DumpHeap;
begin
  if ViewHeap = Nil then exit;
  ViewHeap^.Owner^.Delete(ViewHeap);
  Dispose(ViewHeap,Done);
end;

procedure UpdateHeap;
begin
  if ViewHeap = Nil then exit;
  ViewHeap^.Update;
end;

{----------------------------------------------------}
{ From SHAZAM II: CLOCK2.INC }
procedure AddClock;
var
  R: TRect;
begin
  if Application = Nil then exit;
  Application^.GetExtent(R);
  R.A.X:= R.B.X - 17;
  R.B.Y:= R.A.Y + 1;
  Clock:= New(PClockView, Init(R));
  Application^.Insert(Clock);
end;

procedure DumpClock;
begin
  if Clock = Nil then exit;
  Clock^.Owner^.Delete(Clock);
  Dispose(Clock, Done);
end;

procedure UpdateClock;
begin
  if Clock = Nil then exit;
  Clock^.Update;
end;

{----------------------------------------------------}
procedure SetDesktop;
begin
  with DeskViewRec do begin
    if roVideo then VideoOptions := 1 else VideoOptions := 0;
    ColorOptions := roColor;
    DeskOptions := 0;
    if roClock > 0 then DeskOptions := DeskOptions or (roClock * 2);
    if roHeap then DeskOptions := DeskOptions or $01;
    DateOptions := roDate;
    SeparatorsOptions := roSep - 1;
  end;
end;

procedure LetHeapClock;
begin
  roHeap := (DeskViewRec.DeskOptions and $1) = 1;
  roClock := (DeskViewRec.DeskOptions and $06) div 2;
end;

procedure LetColor;
begin
  case roColor of
    0 : AppPalette := apColor;
    1 : AppPalette := apBlackWhite;
    2 : AppPalette := apMonochrome;
  end;
  DoneMemory;
  Application^.Redraw;
end;

procedure hdDeskView;
begin
  SetDeskTop;
  if Application^.ExecuteDialog(MakeDialog, @DeskViewRec) = cmOK then
    with DeskViewRec do begin
      LetHeapClock;
      if roVideo <> (VideoOptions = 1) then begin
        roVideo := (VideoOptions = 1);
        Application^.SetScreenMode(ScreenMode xor smFont8x8);
        LetColor;
      end;
      if ColorOptions <> roColor then begin
        roColor := ColorOptions;
        LetColor;
      end;
      if roDate <> DateOptions then roDate := DateOptions;
      if roSep <> (SeparatorsOptions + 1) then roSep := SeparatorsOptions + 1;
      UpdateHeap;
      UpdateClock;
    end;
end;

begin
  roVideo := (ScreenMode > 7);  { actual mode }
  roColor := AppPalette;  { actual color palette }
  SetDesktop;
end.


{****************************************************************************

   Portions Copyright

- Turbo Vision, Turbo Pascal, original Gadgets:
      (c) 1992 by Borland International
- the Clock outline:
      (c) 1991,92 by Johnathan J. Stein
      from Shazam II clock.inc procedures
- the heap outline:
      (c) 1991 by Addison-Wesley (Deutschland) GmbH
      from Ertl, Machhholz, Schallmaier (1991) Turbo Pascal 6.0 Turbo Vision:
      Konzepte, Programmierung, Anwendungen
- the Desktop Options dialog idea:
      (c) 1993 by Que Corporation
      from Mitchell (1993) Borland(r) Pascal Developer's Guide, p.488
- Dialog Design:
      (c) 1990-4 by L. David Baldwin

 **************************************************************************

History:

clocheap.pas
Revision 1.6  1995-01-22 16:24:33-0800  Germano
Added APP.DEF; corrected some minor bugs.

Revision 1.5  1994-11-12 23:40:33-0800  Germano
Added SaverScreen by Groothuis.

Revision 1.4  1994-11-08 09:25:54-0800  Germano
Added TSelLabel, TSelCheckBoxes and TSelRadioButtons.

Revision 1.3  1994-11-08 01:21:12-0800  Germano
Added TClockHeapApplication, with Clock and Heap capacities.

Revision 1.2  1994-09-10 08:42:51-0700  Germano
Header changes in order to use with RCS.

Revision 1.1  1994-09-02 23:51:46-0700  Germano
Initial revision

**************************************************************************)
